# Generated by Django 2.2.23 on 2021-05-19 13:31

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('django_cog', '0003_task_queue'),
    ]

    operations = [
        migrations.AlterModelOptions(
            name='entityrun',
            options={'ordering': ['-completed_on', '-started_on']},
        ),
        migrations.AddField(
            model_name='task',
            name='weight',
            field=models.DecimalField(decimal_places=5, default=1.0, max_digits=10),
        ),
    ]
