# Generated by Django 2.2.23 on 2021-05-19 12:30

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('django_cog', '0002_auto_20210519_0824'),
    ]

    operations = [
        migrations.AddField(
            model_name='task',
            name='queue',
            field=models.ForeignKey(default=1, null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='assigned_tasks', to='django_cog.CeleryQueue'),
        ),
    ]
