# Generated by Django 2.2.23 on 2021-09-15 19:44

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('django_cog', '0017_auto_20210915_1533'),
    ]

    operations = [
        migrations.AlterField(
            model_name='entityrun',
            name='status',
            field=models.CharField(choices=[('Queued', 'Queued'), ('Running', 'Running'), ('Failed', 'Failed'), ('Completed', 'Completed'), ('Canceled', 'Canceled')], default='Queued', max_length=10, null=True),
        ),
    ]
