# Generated by Django 2.2.23 on 2021-09-14 18:48

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('django_cog', '0007_auto_20210525_0810'),
    ]

    operations = [
        migrations.AddField(
            model_name='task',
            name='critical',
            field=models.BooleanField(default=True),
        ),
    ]
