# Generated by Django 2.2.23 on 2021-05-25 12:10

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('django_cog', '0006_auto_20210525_0809'),
    ]

    operations = [
        migrations.AlterField(
            model_name='pipeline',
            name='schedule',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, to='django_celery_beat.CrontabSchedule'),
        ),
    ]
