# Generated by Django 2.2.23 on 2021-05-19 12:24

from django.db import migrations, models


def create_celery_queue(apps, schema_editor):
    """
    Creates a default "celery" queue.  This is what the next
    migration will use as the default.
    """
    CeleryQueue = apps.get_model('django_cog', 'CeleryQueue')
    CeleryQueue.objects.create(
        id=1,
        queue_name='celery'
    )

class Migration(migrations.Migration):

    dependencies = [
        ('django_cog', '0001_initial'),
    ]

    operations = [
        migrations.CreateModel(
            name='CeleryQueue',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('queue_name', models.CharField(max_length=4096)),
            ],
        ),
        migrations.AlterModelOptions(
            name='taskrun',
            options={'ordering': ['completed_on']},
        ),
        migrations.RunPython(create_celery_queue)
    ]
