# Generated by Django 2.2.23 on 2021-05-19 14:06

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('django_cog', '0004_auto_20210519_0931'),
    ]

    operations = [
        migrations.AlterField(
            model_name='task',
            name='weight',
            field=models.DecimalField(decimal_places=5, default=1.0, max_digits=12),
        ),
    ]
