# Generated by Django 2.2.23 on 2021-09-15 13:17

from django.db import migrations, models
import uuid


class Migration(migrations.Migration):

    dependencies = [
        ('django_cog', '0010_auto_20210914_1500'),
    ]

    operations = [
        migrations.CreateModel(
            name='CogErrorHandler',
            fields=[
                ('id', models.UUIDField(default=uuid.uuid4, editable=False, primary_key=True, serialize=False)),
                ('created_date', models.DateTimeField(auto_now_add=True)),
                ('modified_date', models.DateTimeField(auto_now=True)),
                ('name', models.CharField(max_length=4096)),
            ],
            options={
                'abstract': False,
            },
        ),
    ]
