# Generated by Django 2.2.23 on 2021-09-15 14:42

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('django_cog', '0015_cogerror_error_type'),
    ]

    operations = [
        migrations.AlterField(
            model_name='cogerror',
            name='timestamp',
            field=models.DateTimeField(auto_created=True, auto_now_add=True),
        ),
    ]
